@@ -15,6 +15,7 @@ const (
 	ADTS_FRAME          = "adts_frame"
 	APEV2               = "apev2"
 	AR                  = "ar"
+	ASN1_BER            = "asn1_ber"
 	AV1_CCR             = "av1_ccr"
 	AV1_FRAME           = "av1_frame"
 	AV1_OBU             = "av1_obu"
@@ -25,6 +26,7 @@ const (
 	AVC_PPS             = "avc_pps"
 	AVC_SEI             = "avc_sei"
 	AVC_SPS             = "avc_sps"
+	AVRO_OCF            = "avro_ocf"
 	BENCODE             = "bencode"
 	BSD_LOOPBACK_FRAME  = "bsd_loopback_frame"
 	BSON                = "bson"
