@@ -55,7 +55,7 @@ func decodeTCP(d *decode.D, in interface{}) interface{} {
 	d.FieldU16("urgent_pointer")
 	optionsLen := (int64(dataOffset) - 5) * 8 * 4
 	if optionsLen > 0 {
-		d.LenFn(optionsLen, func(d *decode.D) {
+		d.FramedFn(optionsLen, func(d *decode.D) {
 			d.FieldArray("options", func(d *decode.D) {
 				for !d.End() {
 					d.FieldStruct("option", func(d *decode.D) {
