@@ -9,6 +9,7 @@ import (
 
 	"github.com/wader/fq/pkg/bitio"
 	"github.com/wader/fq/pkg/ranges"
+	"github.com/wader/fq/pkg/scalar"
 )
 
 type Compound struct {
@@ -26,7 +27,7 @@ type Value struct {
 	V          interface{} // scalar.S or Compound (array/struct)
 	Index      int         // index in parent array/struct
 	Range      ranges.Range
-	RootBitBuf *bitio.Buffer
+	RootReader bitio.ReaderAtSeeker
 	IsRoot     bool // TODO: rework?
 }
 
@@ -224,3 +225,20 @@ func (v *Value) postProcess() {
 		panic(err)
 	}
 }
+
+func (v *Value) TryScalarFn(sms ...scalar.Mapper) error {
+	var err error
+	sr, ok := v.V.(*scalar.S)
+	if !ok {
+		panic("not a scalar value")
+	}
+	s := *sr
+	for _, sm := range sms {
+		s, err = sm.MapScalar(s)
+		if err != nil {
+			break
+		}
+	}
+	v.V = &s
+	return err
+}
